#
# Fixture collection for pytest
#

#
# command line arguments
# ----------------------
#
# see
#   https://docs.pytest.org/en/latest/example/simple.html#pass-different-values-to-a-test-function-depending-on-command-line-options
#

import pytest
import requests
import urllib
import tempfile
import zipfile
import json
import shutil
import os
import schul_cloud_ressources_api_v1.auth as auth
from schul_cloud_ressources_api_v1.rest import ApiException
from schul_cloud_ressources_api_v1 import ApiClient, RessourceApi
from schul_cloud_ressources_api_v1.schema import get_valid_examples, get_invalid_examples
from schul_cloud_ressources_server_tests.tests.fixtures import *


NUMBER_OF_VALID_RESSOURCES = 3
NUMBER_OF_INVALID_RESSOURCES = 2
RESSOURCES_API_ZIP_URL = "https://github.com/schul-cloud/ressources-api-v1/archive/master.zip"
RESSOURCES_EXAMPLES_BASE_PATH = "ressources-api-v1-master/schemas/ressource/examples"


@pytest.fixture
def valid_ressources():
    """Return a list of valid ressoruces useable by tests."""
    return get_valid_examples()


@pytest.fixture
def invalid_ressources():
    """Return a list of invalid ressoruces useable by tests."""
    return get_invalid_examples()


@pytest.fixture
def a_valid_ressource(valid_ressources):
    """Return a valid ressource.

    This fixture is not parametrized and does not mulitply the tests.
    """
    return valid_ressources[0]


# https://docs.pytest.org/en/latest/fixture.html#parametrizing-fixtures
@pytest.fixture(params=list(range(NUMBER_OF_VALID_RESSOURCES)))
def valid_ressource(request, valid_ressources):
    """Return a valid ressource."""
    return valid_ressources[request.param % len(valid_ressources)]


@pytest.fixture(params=list(range(NUMBER_OF_INVALID_RESSOURCES)))
def invalid_ressource(request, invalid_ressources):
    """Return an invalid ressource."""
    return invalid_ressources[request.param % len(invalid_ressources)]


def pytest_addoption(parser):
    """Add options to pytest.

    This adds the options for
    - url to store the value
    - token to add the token to a list
    - basic to add the credentials to a list
    - noauth if you do not want to test without authentication
    """
    parser.addoption("--url", action="store", default="http://localhost:8080/v1/",
        help="url: the url of the server api to connect to")
    parser.addoption("--noauth", action="store", default="true",
        help="noauth: whether to connect without authentication")
    parser.addoption("--basic", action="append", default=[],
        help="basic: list of basic authentications to use")
    parser.addoption("--apikey", action="append", default=[],
        help="apikey: list of api key authentications to use")

ERROR_BASIC = "user name and password must be divided by \":\" when "\
              "using --basic=username:password as a test parameter"

def get_credentials(metafunc):
    """Generate a list of credentials from the test parameters.

    Return a tuple (authentication mechanism, user name, password)
    """
    users = []
    if metafunc.config.option.noauth == "true":
        users.append(("noauth", None, None))
    for credentials in metafunc.config.option.basic:
        assert ":" in credentials, ERROR_BASIC
        username, password = credentials.split(":", 1)
        users.append(("basic", username, password))
    for credentials in metafunc.config.option.apikey:
        credentials = credentials.split(":", 1)
        username = (["anonymous api key"] if len(credentials) == 1 else credentials[0])
        users.append(("apikey", username, credentials[-1]))
    return users


def pytest_generate_tests(metafunc):
    """Generate parameters.

    The authentication parameters require special handling
    to create nice test cases.
    """
    if "all_credentials"  in metafunc.fixturenames:
        metafunc.parametrize("all_credentials", [get_credentials(metafunc)])
    if "_user2" in metafunc.fixturenames and \
        "_user1_auth2" in metafunc.fixturenames:
        raise NotImplementedError()
    elif "_user1" in metafunc.fixturenames and \
        "_user2" in metafunc.fixturenames:
        credentials = get_credentials(metafunc)
        params = [(u1, u2) for u1 in credentials for u2 in credentials
                  if u1[1] != u2[1]]
        metafunc.parametrize("_user1,_user2", params)
    elif "_user1" in metafunc.fixturenames and \
        "_user1_auth2" in metafunc.fixturenames:
        credentials = get_credentials(metafunc)
        params = [(u1, u2) for u1 in credentials for u2 in credentials
                  if u1[1] == u2[1] and u1[2] != u2[2]]
        metafunc.parametrize("_user1,_user1_auth2", params)
    elif "_user1" in metafunc.fixturenames:
        credentials = get_credentials(metafunc)
        metafunc.parametrize("_user1", credentials)
    if "_invalid_user" in metafunc.fixturenames:
        credentials = get_credentials(metafunc)
        invalid_credentials = [
            ("apikey", "", ""), # empty api key
            ("basic", "", ""), # empty username and password
        ]
        if not ("noauth", None, None) in credentials:
            invalid_credentials.append(("noauth", None, None))
        for cred in credentials:
            if cred[0] == "basic":
                # empty password or user name
                invalid_credentials.append(("basic", cred[1], ""))
                invalid_credentials.append(("basic", "", cred[2]))
                invalid_credentials.append(("basic", "invalid" + cred[1], cred[2]))
                invalid_credentials.append(("basic", cred[1], "invalid" + cred[2]))
                invalid_credentials.append(("basic", cred[2], cred[1]))
                invalid_credentials.append(("apikey", None, cred[1]))
                invalid_credentials.append(("apikey", None, cred[2]))
            elif cred[0] == "apikey":
                invalid_credentials.append(("apikey", None, cred[1]))
                invalid_credentials.append(("basic", cred[2], cred[1]))
                invalid_credentials.append(("basic", cred[1], cred[2]))
                invalid_credentials.append(("apikey", None, cred[1] + "invalid"))
                invalid_credentials.append(("apikey", None, cred[2] + "invalid"))
                invalid_credentials.append(("basic", cred[2], cred[1] + "invalid"))
                invalid_credentials.append(("basic", cred[1], cred[2] + "invalid"))
                invalid_credentials.append(("basic", "invalid" + cred[2], cred[1]))
                invalid_credentials.append(("basic", "invalid" + cred[1], cred[2]))
        metafunc.parametrize("_invalid_user", invalid_credentials)


class User(object):
    """The user object for the tests.

    The user has an api which uses a certain authentication.
    """

    def __init__(self, api, auth_type, name, secret):
        """Create a new user object."""
        self._api = api
        assert auth_type in ["noauth", "basic", "apikey"]
        self._auth_type = auth_type
        self._name = name
        self._secret = secret
    
    @property
    def name(self):
        """The user name, None if no name is given."""
        return self._name

    @property
    def credentials(self):
        """The authentication credentials, None if none are given."""
        return self._name, self._secret

    def authenticate(self):
        """Authenticate the user."""
        if self._auth_type == "noauth":
            auth.none()
        elif self._auth_type == "basic":
            auth.basic(self._name, self._secret)
        elif self._auth_type == "apikey":
            auth.api_key(self._secret)
        else:
            raise ValueError(self._auth_type)

    @property
    def api(self):
        """Return an api object that is authenticated."""
        self.authenticate()
        return self._api

    def __repr__(self):
        """A string representation."""
        return "User(api, {}, {}, {})".format(self._auth_type, repr(self._name), repr(self._secret))


@pytest.fixture
def user1(_user1, _api):
    """Return a user for the api with credentials."""
    return User(_api, *_user1)


@pytest.fixture
def user1_auth2(_user1_auth2, _api):
    """Return a user for the api with credentials."""
    return User(_api, *_user1_auth2)


@pytest.fixture
def user2(_user2, _api):
    """Return a user for the api with credentials."""
    return User(_api, *_user2)


@pytest.fixture
def invalid_user(_invalid_user, _api):
    """Return an invalid user."""
    return User(_api, *_invalid_user)


@pytest.fixture
def url(request):
    """The url of the server."""
    return request.config.getoption("--url").rstrip("/")


@pytest.fixture
def client(url):
    """The client object connected to the API."""
    return ApiClient(url)


@pytest.fixture
def _api(client):
    """The api to use to test the server."""
    return RessourceApi(client)


@pytest.fixture
def api(user1):
    """The api uses the authentication credentials."""
    return user1.api


def step(function):
    """Allow pytest -m stepX to run test up to a certain number."""
    step_number = len(_steps) + 1
    step_only_marker = "step{}only".format(step_number)
    marker_only = getattr(pytest.mark, step_only_marker)
    step_marker = "step{}".format(step_number)
    marker = getattr(pytest.mark, step_marker)
    def mark_function(marker):
        marker(function)
    for mark_step in _steps:
        mark_step(marker)
    _steps.append(mark_function)
    return marker_only(marker(function))
_steps = []

__builtins__["step"] = step



import requests


def test_server_is_there(ressources_server):
    """Test that the server is there."""
    requests.get(ressources_server.url)


def test_server_works_on_data(ressources_server, valid_ressource):
    """Test that the api adds a ressource."""
    ressources_server.api.add_ressource(valid_ressource)
    assert ressources_server.get_ressources() == [valid_ressource]


# this must be the last test
def test_server_stops(ressources_server):
    """Test that the server stops in the end."""
    requests.get(ressources_server.url)
    


import requests
from schul_cloud_ressources_api_v1.rest import ApiException
from pytest import raises
import pytest

@step
def test_server_is_reachable(url):
    """There is a server behind the url."""
    result = requests.get(url)
    assert result.status_code


@step
def test_valid_is_not_invalid_ressource(valid_ressource, invalid_ressource):
    """A ressource can not be valid and invalid at the same time."""
    assert valid_ressource != invalid_ressource


@step
def test_add_a_ressource_and_get_id(api, valid_ressource):
    """When adding a ressource, we should get an id back."""
    result = api.add_ressource(valid_ressource)
    assert isinstance(result.id, str)


@step
def test_add_a_ressource_and_retrieve_it(api, valid_ressource):
    """When we save a ressource, we should be able to get it back."""
    result = api.add_ressource(valid_ressource)
    copy = api.get_ressource(result.id)
    assert valid_ressource == copy


@step
def test_there_are_at_least_two_valid_ressources(valid_ressources):
    """For the next tests, we will need two distinct valid resssources."""
    assert len(valid_ressources) >= 2


@step
def test_add_two_different_ressources(api, valid_ressources):
    """When we post two different ressources, we want the server to distinct them."""
    r1 = api.add_ressource(valid_ressources[0])
    c1_1 = api.get_ressource(r1.id)
    r2 = api.add_ressource(valid_ressources[1])
    c2_1 = api.get_ressource(r2.id)
    c1_2 = api.get_ressource(r1.id)
    c2_2 = api.get_ressource(r2.id)
    c1_3 = api.get_ressource(r1.id)
    assert c1_1 == valid_ressources[0], "see test_add_a_ressource_and_retrieve_it"
    assert c1_2 == valid_ressources[0]
    assert c1_3 == valid_ressources[0]
    assert c2_1 == valid_ressources[1]
    assert c2_2 == valid_ressources[1]


@step
def test_deleted_ressource_is_not_available(api, valid_ressource):
    """If a client deleted a ressource, this ressource should be absent afterwards."""
    r1 = api.add_ressource(valid_ressource)
    api.delete_ressource(r1.id)
    with raises(ApiException) as error:
        api.get_ressource(r1.id)
    assert error.value.status == 404


@step
def test_list_of_ressource_ids_is_a_list(api):
    """The list returned by the api should be a list if strings."""
    ids = api.get_ressource_ids()
    assert all(isinstance(_id, str) for _id in ids)


@step
def test_new_resources_are_listed(api, valid_ressource):
    """Posting new ressources adds them their ids to the list of ids."""
    ids_before = api.get_ressource_ids()
    r1 = api.add_ressource(valid_ressource)
    ids_after = api.get_ressource_ids()
    new_ids = set(ids_after) - set(ids_before)
    assert r1.id in new_ids


@step
def test_ressources_listed_can_be_accessed(api):
    """All the ids listed can be accessed."""
    ids = api.get_ressource_ids()
    for _id in ids[:10]:
        api.get_ressource(_id)


@step
def test_delete_all_ressources_removes_ressource(api):
    """After all ressources are deleted, they can not be accessed any more."""
    ids = api.get_ressource_ids()
    api.delete_ressources()
    for _id in ids[:10]:
        with raises(ApiException) as error:
            api.get_ressource(_id)
        assert error.value.status == 404


@step
def test_delete_ressources_deletes_posted_ressource(api, valid_ressource):
    """A posted ressource is deleted when all ressources are deleted."""
    r1 = api.add_ressource(valid_ressource)
    api.delete_ressources()
    with raises(ApiException) as error:
        api.get_ressource(r1.id)
    assert error.value.status == 404


@step
def test_delete_ressource_can_not_be_found_by_delete(api, valid_ressource):
    """When a ressource is deleted, it can not be found."""
    r1 = api.add_ressource(valid_ressource)
    api.delete_ressource(r1.id)
    with raises(ApiException) as error:
        api.delete_ressource(r1.id)
    assert error.value.status == 404


@step
def test_there_are_invalid_ressources(api, invalid_ressources):
    """Ensure that the tests have invalid ressources to run with."""
    assert invalid_ressources


@step
def test_unprocessible_entity_if_header_is_not_set(url):
    """If the Content-Type is not set to application/json, this is communicated.

    https://httpstatuses.com/415
    """
    response = requests.post(url + "/ressources", data="{}")
    assert response.status_code == 415



@step
def test_bad_request_if_there_is_no_valid_json(url):
    """If the posted object is not a valid JSON, the server notices it.

    https://httpstatuses.com/400
    """
    response = requests.post(url + "/ressources", data="invalid json", 
                             headers={"Content-Type":"application/json"})
    assert response.status_code == 400


@step
def test_invalid_ressources_can_not_be_posted(api, invalid_ressources):
    """If the ressources do not fit in the schema, they cannot be posted.

    The error code 422 should be returned.
    https://httpstatuses.com/422
    """
    for invalid_ressource in invalid_ressources:
        with raises(ApiException) as error:
            api.add_ressource(invalid_ressource)
        assert error.value.status == 422, "Unprocessable Entity"


@step
def deleting_and_adding_a_ressource_creates_a_new_id(api, valid_ressorce):
    """When a ressource is added and deleted and added,
    the id of existing ressources are not taken."""
    ids = [api.add_ressource(valid_ressource).id for i in range(4)]
    api.delete_ressource(ids[1])
    r = api.add_ressource(valid_ressource)
    assert r.id not in ids


@step
def test_posting_and_deleting_a_ressource_leaves_other_ressource_intact(
        api, valid_ressources):
    """When several ressoruces are posted, they are left intact if reposted."""
    ids = [api.add_ressource(valid_ressources[0]).id for i in range(4)]
    api.delete_ressource(ids.pop(2))
    r = api.add_ressource(valid_ressources[1])
    for _id in ids:
        ressource = api.get_ressource(_id)
        assert ressource == valid_ressources[0]

ERROR_NEED_MORE_CREDENTIALS = "Please pass additional authentication arguments."


class TestAuthentication:
    """Test the authentication mechanism.

    This includes:

    - different users can not see eachother
    - same users can post with different authentication machanisms
    - unauthenticated user can not see anything from other users
    - invalid username, password, api key
    - malformed Authorize header
    - empty username, password, api key
    """

    @step
    def test_user_provided_credentials(self, all_credentials):
        """To make the tests work, the user should provide credentials."""
        assert len(all_credentials) >= 2, ERROR_NEED_MORE_CREDENTIALS

    @step
    class TestTest:
        """Test the tests"""

        def test_users_are_always_unequal(self, user1, user2):
            """user1 and user2 are never the same."""
            assert user1.name != user2.name
            assert user1.credentials != user2.credentials

        def test_different_auths_have_the_same_name(self, user1, user1_auth2):
            """user1 is the same user as user1_auth2 but uses a different
            authentication mechanism"""
            assert user1.name == user1_auth2.name
            assert user1.credentials != user1_auth2.credentials

    @step
    def test_cannot_cross_post(self, user1, user2, a_valid_ressource):
        """Two disjoint users can not access each others objects."""
        ids = user2.api.get_ressource_ids()
        for i in range(len(ids) + 1):
            r = user1.api.add_ressource(a_valid_ressource)
            if r.id not in ids:
                break
        else:
            assert False, "This isnot expected."
        ids = user2.api.get_ressource_ids()
        assert r.id not in ids, "{} and {} must not access the same resources.".format(user1, user2)

    @step
    def test_cannot_cross_delete(self, user1, user2, a_valid_ressource):
        """Two users can not delete each other's ressources."""
        r = user1.api.add_ressource(a_valid_ressource)
        try:
            user2.api.delete_ressource(r.id)
        except ApiException:
            pass
        ressource = user1.api.get_ressource(r.id)
        assert ressource == a_valid_ressource

    def assertSameIds(self, user1, user1_auth2):
        """Make sure the ids are the same."""
        ids1 = user1.api.get_ressource_ids()
        ids2 = user1_auth2.api.get_ressource_ids()
        assert ids1 == ids2

    @step
    def test_same_user_can_view_ressources_with_different_auth(
            self, user1, user1_auth2, a_valid_ressource):
        """Regardless of the authentication mechanism, the user can view 
        the ressource."""
        self.assertSameIds(user1, user1_auth2)
        r = user1.api.add_ressource(a_valid_ressource)
        self.assertSameIds(user1, user1_auth2)
        user1_auth2.api.delete_ressource(r.id)
        self.assertSameIds(user1, user1_auth2)

    @step
    @pytest.mark.parametrize("action", [
            lambda api, res: api.add_ressource(res),
            lambda api, res: api.get_ressource_ids(),
            lambda api, res: api.delete_ressource("64682437"),
            lambda api, res: api.get_ressource("tralala"),
            lambda api, res: api.delete_ressources()
        ])
    def test_invalid_user_can_not_access_the_api(
            self, action, invalid_user, a_valid_ressource):
        """The invalid user gets a 401 unauthorized all the time."""
        with raises(ApiException) as error:
            action(invalid_user.api, a_valid_ressource)
        assert error.value.status == 401

    @step
    @pytest.mark.parametrize("header", [
            "api-key yek=aaaaaa", "api-key",
            "blablabla", "api-key key=aaaaaa,asd=asd"
        ])
    def test_unspecified_authorization_headers_yield_401(self, url, header):
        """"""
        result = requests.get(url + "/ressources/ids",
                              headers={"Authorization": header})
        assert result.status_code == 401



#!/usr/bin/python3
import sys
import json
import jsonschema
import base64
from schul_cloud_ressources_api_v1.schema import validate_ressource, ValidationFailed
from bottle import request, abort, response, tob, touni, Bottle

app = Bottle()

run = app.run
post = app.post
get = app.get
delete = app.delete

# configuration constants
BASE = "/v1"

# global variables
last_id = 0


class data(object):
    """The data interface the server operates with."""

    @staticmethod
    def delete_ressources():
        """Initialize the ressources."""
        global _ressources
        _ressources = {
            "valid1@schul-cloud.org": {},
            "valid2@schul-cloud.org": {},
            None: {}
        } # user: id: ressource

    @staticmethod
    def get_ressources():
        """Return all stored ressources."""
        ressources = []
        for user_ressources in _ressources.values():
            ressources.extend(user_ressources.values())
        return ressources


data.delete_ressources()

passwords = {
    "valid1@schul-cloud.org": "123abc",
    "valid2@schul-cloud.org": "supersecure"
}
api_keys = {
   "abcdefghijklmn": "valid1@schul-cloud.org"
}

HEADER_ERROR = "Malfomred Authorization header."

def get_api_key():
    """Return the api key or None."""
    header = request.headers.get('Authorization')
    if not header: return
    try:
        method, data = header.split(None, 1)
        if method.lower() != 'api-key': return
        return touni(base64.b64decode(tob(data[4:])))
    except (ValueError, TypeError):
        abort(401, HEADER_ERROR) 

BASIC_ERROR = "Could not do basic authentication. Wrong username or password."
API_KEY_ERROR = "Could not authenticate using the given api key."

def get_ressources():
    """Return the ressources of the authenticated user.

    If authentication failed, this aborts the execution with
    401 Unauthorized.
    """
    header = request.environ.get('HTTP_AUTHORIZATION','')
    if header:
        print("Authorization:", header)
    basic = request.auth
    if basic:
        username, password = basic
        if passwords.get(username) != password:
            abort(401, BASIC_ERROR)
    else:
        api_key = get_api_key()
        if api_key is not None:
            username = api_keys.get(api_key)
            if username is None:
                abort(401, API_KEY_ERROR)
        else:
            username = None
    return _ressources[username]


@post(BASE + "/ressources")
def add_ressource():
    """Add a new ressource."""
    global last_id
    ressources = get_ressources()
    _id = str(last_id)
    last_id += 1
    try:
        ressource = request.json
    except (ValueError, TypeError):
        # this can be removed with later releases of bottle
        # https://github.com/bottlepy/bottle/blob/41ed6965de9bf7d0060ffd8245bf65ceb616e26b/bottle.py#L1292
        abort(400, "The request body is not a valid JSON object.")
    if ressource is None:
        abort(415, "JSON is expected.")
    try:
        validate_ressource(ressource)
    except ValidationFailed as error:
        abort(422, error)
    ressources[_id] = ressource
    return {"id" : _id}


@get(BASE + "/ressources/<_id>")
def get_ressource(_id):
    """Get a ressource identified by id."""
    if _id == "ids":
        return get_ressource_ids()
    ressources = get_ressources()
    ressource = ressources.get(_id)
    if ressource is None:
        abort(404, "Ressource not found.")
        return
    return ressource


@delete(BASE + "/ressources/<_id>")
def delete_ressource(_id):
    """Delete a saved ressource."""
    ressources = get_ressources()
    if ressources.pop(_id, None) is None:
        abort(404, "Ressource not found.")


def get_ressource_ids():
    """Return the list of current ids."""
    ressources = get_ressources()
    response.content_type = 'application/json'
    return json.dumps(list(ressources.keys()))


@delete(BASE + "/ressources")
def delete_ressources():
    """Delete all ressources."""
    ressources = get_ressources()
    ressources.clear()


def main():
    """Start the serer from the command line."""
    port = (int(sys.argv[1]) if len(sys.argv) >= 2 else 8080)
    run(host="", port=port, debug=True, reloader=True)


if __name__ == "__main__":
    main()




import pytest
import time
import schul_cloud_ressources_api_v1.auth as auth
from schul_cloud_ressources_server_tests.app import data, app
from schul_cloud_ressources_api_v1 import ApiClient, RessourceApi
from bottle import ServerAdapter
from threading import Thread


class StoppableWSGIRefServerAdapter(ServerAdapter):
    """A bottle adapter for tests which is stoppable.

    copied from bottle
    """

    def run(self, app):
        from wsgiref.simple_server import WSGIRequestHandler, WSGIServer
        from wsgiref.simple_server import make_server
        import socket

        class FixedHandler(WSGIRequestHandler):
            def address_string(self): # Prevent reverse DNS lookups please.
                return self.client_address[0]
            def log_request(*args, **kw):
                if not self.quiet:
                    return WSGIRequestHandler.log_request(*args, **kw)

        handler_cls = self.options.get('handler_class', FixedHandler)
        server_cls  = self.options.get('server_class', WSGIServer)

        if ':' in self.host: # Fix wsgiref for IPv6 addresses.
            if getattr(server_cls, 'address_family') == socket.AF_INET:
                class server_cls(server_cls):
                    address_family = socket.AF_INET6

        self.srv = make_server(self.host, 0, app, server_cls, handler_cls)
        self.srv.serve_forever()
        while not self.get_port(): time.sleep(0.001)

    def shutdown(self):
        """Stop the server."""
        self.srv.shutdown()

    def get_port(self):
        """Return the port of the server."""
        try:
            return self.srv.socket.getsockname()[1]
        except AttributeError:
            return None


class ParallelBottleServer(object):
    """A server that runs bottle in parallel"""

    url_prefix = ""

    def __init__(self, app):
        """Start the server with a bottle app."""
        self._server = StoppableWSGIRefServerAdapter()
        self._thread = Thread(target=app.run, kwargs=dict(host="127.0.0.1", server=self._server))
        self._thread.start()
        while not self._server.get_port(): time.sleep(0.0001)

    @property
    def url(self):
        return "http://localhost:{}{}".format(self._server.get_port(), self.url_prefix)

    def shutdown(self):
        """Shut down the server."""
        self._server.shutdown()
        self._thread.join()



class RessourcesApiTestServer(ParallelBottleServer):
    """Interface to get the objects."""

    url_prefix = "/v1"

    def __init__(self):
        """Create a new server serving the ressources api."""
        ParallelBottleServer.__init__(self, app)

    def get_ressources(self):
        """Return all currently saved ressources."""
        return data.get_ressources()

    def delete_ressources(self):
        """Clean up all ressources."""
        data.delete_ressources()

    @property
    def api(self):
        """An ressources api client connected to the server."""
        auth.none()
        client = ApiClient(self.url)
        return RessourceApi(client)


@pytest.fixture(scope="session")
def session_ressources_server():
    """Return the server to store ressources."""
    session_ressources_server = RessourcesApiTestServer()
    yield session_ressources_server
    session_ressources_server.shutdown()


@pytest.fixture
def ressources_server(session_ressources_server):
    """Return a fresh server object with no ressources."""
    session_ressources_server.delete_ressources()
    yield session_ressources_server
    session_ressources_server.delete_ressources()


__all__  = ["StoppableWSGIRefServerAdapter", "ParallelBottleServer", "RessourcesApiTestServer",
            "session_ressources_server", "ressources_server"]